package sshkeys

import (
	"context"
	"encoding/asn1"
	"encoding/pem"
	"fmt"
	"os"
	"path/filepath"
	"strings"

	"golang.org/x/crypto/ssh"
	"gopkg.in/errgo.v1"

	"github.com/Scalingo/cli/config"
)

var (
	DefaultKeyPath = filepath.Join(config.HomeDir(), ".ssh", "id_rsa")
)

func ReadPrivateKey(ctx context.Context, path string) (ssh.Signer, error) {
	privateKeyContent, err := os.ReadFile(path)
	if err != nil {
		return nil, errgo.Mask(err)
	}
	return readPrivateKeyWithContent(ctx, path, privateKeyContent)
}

func readPrivateKeyWithContent(ctx context.Context, path string, privateKeyContent []byte) (ssh.Signer, error) {
	// We parse the private key on our own first so that we can
	// show a nicer error if the private key has a password.
	block, _ := pem.Decode(privateKeyContent)
	if block == nil {
		return nil, fmt.Errorf("Failed to read key '%s': is not in the PEM format", path)
	}

	privateKey := &PrivateKey{Block: block, Path: path}

	signer, err := privateKey.signer(ctx)
	if err != nil {
		if err, ok := err.(asn1.StructuralError); ok && strings.HasPrefix(err.Msg, "tags don't match") || err.Msg == "length too large" {
			return nil, errgo.Newf("Fail to decrypt SSH key, invalid password.")
		}
		if err, ok := err.(asn1.SyntaxError); ok && err.Msg == "trailing data" {
			return nil, errgo.Newf("The password was OK, but something went wrong.\n" +
				"Please re-run the command with the environment variable DEBUG=1 " +
				"and create an issue with the command output: https://github.com/Scalingo/cli/issues")
		}
		return nil, errgo.Newf("Invalid SSH key or password: %v", err)
	}

	return signer, nil
}
