package sshkeys

import (
	"context"
	"testing"

	"github.com/stretchr/testify/require"
	"golang.org/x/crypto/ssh"
)

var (
	invalidKey                = []byte("OsISHo5VIvELz/bYIK+i04AspqqkDlxzDsStAh\nzguaLoY1eUdDY3+D8mzdcQjLBuNI5CGWMOJuQRrCGGWBaW1BxBxanbPOy8vfIzCG\nlD1qgE09KX+2U64N0hONd00kdHf+wxs7pjWu3gUmMlURPj579qmqZ0mNaSClrDIP\nvQP2Pr35picjsGFWczZE9tXQdl8+CQGdGaPosK+OkUv+hf137E0bYgEnyDFW0hQA\nNaA3uxqUz7gvJyBatPVshm2RdKfU82i7syXfoSkMbFRT6AhGHLlWLBv2iWO5/aWq\nLDFYck1y6J7VFZDtyKbgvaHVc0JvpQvYyquBBzN1OjB8FZ2v/UvC14nuLijf")
	unencryptedRSAKey         = []byte("-----BEGIN RSA PRIVATE KEY-----\nMIIEpAIBAAKCAQEA1fsQE1t928SgDhpRolnOQ13cd6w6yiiPDqms2RsQssZVo8pH\nAVrq9+uhJ7kIHWdhnvULnSF9rHUcr9cBRWHEI6Gh58CCZ6lHETu77Eepn/4EDIQT\nn5OpdG7JStUmHn/5cFm4m4bf56yGIsu7HVdlRaYpu5ioM5dN6w9HndG60VJYWPO2\nIZwP41NRFyGt6Bmeu5uKUKytVUcwoPtiwFkWI59WCO4mt7beXISltd0wQrnfXDgJ\nWE/NLELtUfOOrhpfCrauowd3SnuIUWdTt6AON9TvpH1/YaF+5i9JxRMr+xvI/3uh\nnxuG6BGEAC4sPlVCBTWrQzIxb2I/0tMGiunvFQIDAQABAoIBAEj/FdPUsVHIeOV2\nuwSMaF345h35JjEStwLEhU+m+QJTwKFYfvKKZn/e5SIc8BS/0MCeTbDz3OB5LykZ\nSri1imixWf8qYBIm4DST0R7gr5AB4rW2VM9qWzN6G7cokZF0DM7z/JMp5rYs17GF\nGLcAum6RKclyYWcAJbebhc3j3zxKoCt859b2BMRc+0LAVCxtckhkNZqug8Hegh9u\n2EU5E73l2Bz6Hio1iI+8zNuvZDdxO7ZBxQPYT4DzKx04v3luPcHTcApqB6Mw4/Vy\nVLoMv1EaPlpuQscLfkFEFx8mFkSKzLSEi3sMJ+Oo7MGdJLKf9VZ9c15YeSbxxe5b\nc5BiDaECgYEA9o/Hv1g5/8lG/OEpEI96dieh58Kby+/EYHCUMVK9ioYbD8yLtbfx\nlro3JRtesgGPw2xhVrrFuh06ErBYKgs4YZ11vgKr8ncuYr0HgKjG9ZF4X8LKxahO\nP8nrylnwLClCOHuSo1MinNZgrDmeo1wEOenWbnIf3M6juJKwdaVn/j0CgYEA3iwA\nJoOa4jcLOPkX4CDrS+qR7WiQY4cUukvZtqSijbMYIp2BZu4pwtPPj/buew/OtYR7\n9d5v9pQGahFgnvOuXmOn56OwVR6vjRa2bzdduU2FkpxP7X/0brl8vA7R6cUsX0BZ\nWJ+Woo0Sl2rXyLwn4Pmp/Qcqc4FZAlesSF7gWbkCgYB22siHTJho3TYDIOcSvJdj\n7ab2ebFwlZwsgRhuzlvgMtVIwGl4p7sl1Y5PrOnxFR+gLNafYFmAZiZga/GUEuSS\n6bygY9lPgHQz1LuRIcsHBaDpSJVGwuM1jLjFSqWkJB9v6uG9KY493Hmti/kU9n5z\nDG35v7nsTxBNISk/DQUcYQKBgQDD/MLBNvVpuy7uOmcCc2e+F5ZVfRlqgx4Y3ZKY\nGRHAuI2A87rUZyJ12M3QbP+hlac+3zlPAsN4HMqvxNacQdC/OWBD6lVCMYihiLi0\n0X2fWrCLTdMAaxDgoEZVyykH9MUY3bYzhMyoZpRQ+JKUZM81GmkHacvcCu3PMDIt\nwWxeGQKBgQDJt0P1reUdJbCZs4V8wAhQoQTtpzTOZupLd+Kk2SZCQSsNuPzUOkQn\n/IoAS+O64pdfpaTgC3lwjuPdn+7PWjnk6FpmGhGerrHANxJtord7xz/SorkWrinp\ntKuciH50bVJWIJJOOWO/dRHkMyZlOjiuKIV5DCORdoNSrWp1n8ub7A==\n-----END RSA PRIVATE KEY-----\n")
	aesRSA                    = []byte("-----BEGIN RSA PRIVATE KEY-----\nProc-Type: 4,ENCRYPTED\nDEK-Info: AES-128-CBC,51BA2A89BF5BE8FEEC712AB3EE7406FA\n\nqx131ISIYixc99WVtgNPrhMBlKpZ04UjzBAYSzG1Uib5871DiLGXX+R4PSGopPnG\nu63523VkGjKPZJ6xlPqTavshwhHGH1hWcIWtU4qAD7bgQZU3bGjom4IQslacgCwR\nl3fz3Sz9LzlmTbPtQHdOBTLCqlxh6M7OZ7RwuLsHqaWkVqIX86EQrAe1f6A1SZzW\nCJzV+KBRCIE7lvSSwwwdc8Q2lDItEViF1MKiLF6m0WQIwGxyhfk5/DDu9knJopZH\nl6nO+bPR2kR1alYRf7kiILg9L8OsISHo5VIvELz/bYIK+i04AspqqkDlxzDsStAh\nzguaLoY1eUdDY3+D8mzdcQjLBuNI5CGWMOJuQRrCGGWBaW1BxBxanbPOy8vfIzCG\nlD1qgE09KX+2U64N0hONd00kdHf+wxs7pjWu3gUmMlURPj579qmqZ0mNaSClrDIP\nvQP2Pr35picjsGFWczZE9tXQdl8+CQGdGaPosK+OkUv+hf137E0bYgEnyDFW0hQA\nNaA3uxqUz7gvJyBatPVshm2RdKfU82i7syXfoSkMbFRT6AhGHLlWLBv2iWO5/aWq\nLDFYck1y6J7VFZDtyKbgvaHVc0JvpQvYyquBBzN1OjB8FZ2v/UvC14nuLijf5izP\nrZgEK4f+TijSFIydAk6SF85XNlsCZ8Cp9kjop5YhQ40RDC02eeaezVtAX9PNA6yc\noBKV60Mieu3nGiMvB9macJ4kPXqHAWdUqfAZKj/l6NnidGK7RZlhjiiH9YYtnZng\nHKEWMMqW+OR9ftT+B9xFL/hIJ4aZwS33IwdQ0VOuCBkoRoI+CQiKVXBjC+3xiJ1A\nSBp0nmbhv06A6+lhR2ApFyOhve7YrPEd7B4M+15CuUqOScY7/+NMAzLwK2FmpQo/\nLalRgief8pO8Ojij1I9miq6AEULnVq3VvhCTzabF2cqii0NHT9bnJXKgLEQeFrni\nkmsGp6Q3Tg5Xvdax6Lu0ngovS2Ze1TUA3dnBQXKrinHi8LPldBoUlOwaJD9b2Cgb\nZRm+dpGsNG/FNN3mRsOKkwh769qNkuDLRJWT1rATTh9fxeueEIulfldBtS66I4VP\ntgKMDyu+r+l5UYx2RHFveoW4ZgDC5Kv0n9IIp2Vtz5QQdGB+cGojSKFPubTkHP0b\nuM+u/vDRBgpiyPJm2f7fLjyWlLhnTsdltS/zLgWZwMJXaogAt3Q4Ph7DijHBWJHC\nNJO3XKjuJ7mReyN58pMVOS16NRip+Fy6EE8BtPH2T0b6tdp5S+fjhDE5rFQ+IzLC\nG9SJ7kBlC5j8FjJaH4frhN4NQfa38pnYdpeR+547tvjjZGU9MnqgfXhG9Tw5hoCL\nf8MwR9TWMjVqp1MAsmieG5o8EhZ4zA4+xbAsj85h75oVIQYnn0l5zc23uQMiiJHb\nTKtVhepvely85RARZ5xDUZE3lSIcqv3ZGkEZ+bkZKck/WQwplopWYxQeioSGI/XC\nYfG52cApIyQ3cDTORpag00UjBwsZ9PS9uAtoKA8FIEglPk3KT4lMJSLFwWp8WD8N\nTvXDiz1549EkpkSXvWWsBup+9Z9lWE2ug+3+JewpgY5D643HhRs72hoQBCT5GLhr\n-----END RSA PRIVATE KEY-----\n")
	aesRSAMacOSMaverick       = []byte("-----BEGIN RSA PRIVATE KEY-----\nProc-Type: 4,ENCRYPTED\nDEK-Info: AES-128-CBC,53191A9B04F9144051136E2F3A28137A\n\newkky4snYKKwD+cMvayUPMbco2DfPvX2Pj46TfqPT3wbZpNhuA1pOmsi1+S2A0Wv\nZlVCIMA7Dy9sbRM8OvzQ5f7XJKVjS1ajU6oaiNqImw+B7LYpu3nlxRpCo4OxNSH/\neY8/OpXy6HZxiUkjP/vrBN2sPolTm1qjL8/Gvq2c0JLRdN8ovwAmrX37E7HB0jnz\n3BimC+SNfzZPJTfZlN2tNm9J5Pig8IHuRpufvk3x986LBBmf45wSmmICH+n4qnfV\n41ZJ1Jp0s/FlCUarZxeHEEO4eRBlC4zkum0uxWzqlXKYRUFE7WzksO3HVs6mvL4B\nfSqcocgpQP5y9lIKTaD9TKXjAa2EgDpglbzueiH2kffsBjJP8qqkhicFd2+LP9/g\nY7+oTEByKZNs6Xx0yfgScjxEHXS6QZvKLD2KLgcCQayWPGznIv57Ucg8tq3n4AZu\n1xl+Bpdr6regwk+K32/NCA8JH2KDdgGU4Um9OaFKvUpD/pJ7V4SNlA3Ya6ijY8m6\ncwLBSov3O5+Y2rz7si/gw3QigmP/O1e7MjQGa7UOkdHAW6vyEzhamCyloQ4r4x2B\nYbzgtCh9kMK8V/w1O5bWgvh7iHYwEPoXP/BQJB5BLbIufj7nlptNrewGRVRldCjD\ndwEBFuEmjB5jHGOCCcV3WUJTzfQXzaxICHzkBBF/nuta4HeJbmjizMuDmBnIHdoY\nF/yxTCDug1c9qPwd0g2m6jFLE5i91/In18nHnL1bceuRUp9VsvKtOIP0am/G+DLb\nssmd+F3D6Pq5744caJxpmWWK4UoJMGi4qWvJzV76rzELsi20QILmY+bqNlyiaM3R\nyBuMPDoxrW973qSIwagNslGKLvq73RHEr8eMhhcOWLj9qBSaFpds2vfKvLv5bJ76\n22T93UfFpuy94vgPVDERDwNB2F17PMB0V7bL0Sll82klhABOcLKol6kSRN2uPwpU\nUImOSChh79PMYSV6E/ktf39dQLhb+cQOV4gmTzpjwFcHvlZeVhxeF8ID63r2W017\n1yS9+fKE/FG29lJlIgb32wXECGHT9lAcYXv9hg0vlDstHqHJ7Bj9rtUf/wE07zq1\nEJaQU2FfbyymCnHWZeDp+K1W9X6aNn/y/onKL2jqXwQV9HBdNVAEuDAFkzaBGPX+\ndfTXThp0tvLkKhAfr78UnXmROzKmrtItaOmUrAfvUT13Qr29qm1i0fxtWOCadT8u\no7XUEVgBot8PhfoCQ/JIOyRydG8kI3mCjNDN7G/1iBtaPbU3K0Azh120CuryIwi9\nMe/RXxl7EnDRxrVh1vED1MXk5XhK4lUuvHEDQfOLZcKjzLl786iuARPXrsuwCyDd\nR51z8gHjML10pmyaLRiI2VJ7P8Y3qFlnz9AOTiPXXb7tsPDQVGIsEBVa78a2ujJz\nFdsFZGDmDet7kblnfb7b+33a2BA3PceZcH/4hFZUP20gH55GGuQiswJS0z2HqKcu\n08Wwg/x7Iuzj48JPeGQ/DbpQxepyf0nO1V2cMTXLWng1JJuSV182ONd6PKQB2fCs\n3plwPFiPcE1mvvj+/5usEIU8yg5IoUHuu+Pxa8F5Jka2n6D6xHKQTzKSwt0ZZstS\n-----END RSA PRIVATE KEY-----\n")
	aesRSAMacOSYosemite       = []byte("-----BEGIN RSA PRIVATE KEY-----\nProc-Type: 4,ENCRYPTED\nDEK-Info: AES-128-CBC,B53AA8CF8AB3E2F753B0854CD84295EB\n\nV8X2dAVpxr6UKmAZ/nKmRySRvW1YR9UategSDSVKD0Xy2bd9I0hPrDhEcqVpiGly\nPy2vZb6j4N0F8Duh7mNoT8BVVbB7TEmprsMgmMUqw4lGObd5qLq4vEJkkbn1lmdL\nLp1A3DCENDcpyHvOYgo/3XV93lKWHhZ3uF7GzTrGhkdoQuYxRUAAUTKp8gyOoP7n\nnJmYGuy9h+/vlmn9JC3ZJjs5257fY10de61KQpvXIepzKABWTSGCa+ljGb4SVQc1\nClhzaizbkVy/yH6XNYWwfChpb5ooKwr60BQ5OpeYPeuG2YjwgPiG/SXE/H+cU9Bg\n3EbaTcfXdlnkBpVx/ssWp57zJF2PDeA+xM4zxkNMwDtGdkeIDTtdT02mgvTIA5BU\nxCUYw3OCK3blPTIDFxZv1VJbL63NFg0JDf0NQNkIjg0VTAwMluVuvzwcHoQVjvU7\nQlZVJ+huxpyYSQq9nIv/NFpYMBqk/WlkYnUkFRUAGTZe6BLcxKgpG+Jk+NAPK6Wj\nC4hetFC0hy+UrlEYf0BGSmltIUyV536DZHHQdLeadRNl8cYxPAoXCSyADlPLumPp\nEzA5AYCy8ZwX4SiAKfYBRHOhqMqq6NejJ8UEZ1K8Q7fdi677mbROslvS5+u+RYYe\nfZ5WgaOI6WS0asiXstEs4JSOjqNcOZk2nqmsY3Hop9Trnx9TpCqB5XFag/xUKoeo\nnhOnGcvgv1hc2zKgbzqZ0PgBvSYKZoQW+z9dKOJSnp4yoanbmFMKfQJ7iB7rP380\nEipFUxF5bzaWwbfoJZ6GUpC0irH5nhSXerqzJxAgCD1mxOr7aNciC+IqT57hw1mj\nc/ggU5dCIsu41jdJvB8ejVfJbrEVmBFrBpo1ZH49SFbRmd+tn8Xah5SqFmXgiIGY\nNk65rYO7FUH6Owvsokmy4+Xki87mKfDYbZsAYYImdUEEKbaUYtv1qcBhQpZ9iYdK\n8jCygDW1SYm2uTa8G7x015Py0uxOyns7HYLqZZoXFDpw6RdRC+gf2HaGKA8Wz5Jf\ndYqGXs2T0AuAkgzwmsJgHwLUW96Ps5hWYzPDbgN0OcVEmH+hLR7LUFvJp/am4iDo\n8rKFCKSUuy2k/oHQSHYMoyqB3gjigljXUM3briDe+pSgL5UZYgfc+2LLL52OM/mS\nz3a4Z0Hx3C88WzlIYBTO2mLJzmq5Iiu464g1S4iYAQly7iDTOaxoVuH+UJ6IYBXH\nzpKjv3poEYHx9Rvm7asnX04P45ndMiRyjLFVIR2mYQ/HisafsuJmWzVN/mN3t361\nMge9yad6JX17ufqInjGNJNRUz3t5/WkJQYAMJ9orRtyyTSZ3ho5hmP5tXtEmseeR\nMSa+dO5FnJG7S7nRL3pNLtbzd0NIX4r3GeZnXVWFeGWzKPyuDWfatb7aBbY1QMtq\n49xDpZnxJTkCVKjgeBqcx9hJhHPd+7vA9r27Afvp+Z9IYHMcUtzkDd2TqeJBAX7i\nUfoyCE34CSWcrz2yCZ7ezZQBFprbBqKguna1Et391sXolvvbvGshfIMPJ4euYyCx\nc8FUz8ww65I090tsyAnJ4R8iItiEAYokJOoAjanzhcquZ2F302Havlpc+wvj2P32\n-----END RSA PRIVATE KEY-----\n")
	des3RSA                   = []byte("-----BEGIN RSA PRIVATE KEY-----\nProc-Type: 4,ENCRYPTED\nDEK-Info: DES-EDE3-CBC,37FA1FA0BB419924\n\nWMR2lZoviyDLlb6FW1aTuaPeGJ4hTEPbGpqHUjLtXTPdwj5cv3SL5Q4Cx+LdTAuF\nswhhakr+OuL5E9nRTkE6sQKHC9Y4y5hq2M0DM3uyy2tLa6hXLyP+GoEKRcSHYBq6\nZyKpFViM667NEmZuXLT8VOd7xOuAyNO/vJ+zoWThZNn8EkrudKhguSDTpC0S7ZVa\nj42iUoAQA949W2RJyGukfTwsjPA7b09I7IVHOxsRqXjf4XafRILSUlyL6Pe1NHk8\nznQIjvuIVSzwkYyqj3e4qkZwlLXFAgqR39O8b1ECF9fvkFEWwMzggs6DYlHDgjHk\nRElK3PYNzRDTVsATwJxvi9mESWFXfTv6PAi0Lyoyoo5igfeWbOQhZVjvfbTondqq\nDiC0Bh1t9qr7dHmIkkzVBUzEKvAati4tmH4MJScYoCRcDYaIBx2HF73MCWHh3bPG\nWwBAfi3PSSEPEN0JqEM7/P63PjhVS6Uq+Zyfqo40eMnwTBGVilkREjRx7xIb1xPr\no5vBYbzrOoP93w6Cqfy/Fp6jLerySFEt+B3nh3GtzmumWs5v31ru30JvA+kURUM7\ne7Zqtgl2nHG2ap67wZ/ri8tFqB5AggHuLmbxyBJDd6z+glzxVfu74NhV4jQDApr4\nNLe7ewbPZxWeXTh40f/EltK6Yh5ZZoMvs6jZraGsiuau2N6wVv9CejYFmISRo8QG\n3Tnu5MpC2PIyY9xz3bsR/jRd4PQenYjgyEJQZxbhRDy6L3Yp0lSQSgDcrspXPkKy\nioP+1A5Oakf52F8gjLO+lRGMYSxHrPZl4pFHkXL7Yv6A355nO2ZArDhLuy/HvQsg\nhbN7q0H/yROJp/63qvOddl+yBmgm7vGIzdOIwtP5tqEvadaXwiNrzMWBGnQkVZnZ\nnahpoZwDWGAz/6l7u1gAP4gYBB/I6fOli4SF5gHmx9kMB/oL/WGozFo4Xeq6ayWM\nt+D2UxoSkLzRtL73Zbvs3geukvrN5XwiQUdVAjhGLg9uv7Ngcwg5TsIhAJEbN+y6\n4pOh7xmppxNJJYOzgoImwF9l3NnGIPdwwJZK1lBcJMdEliCxyo9lp4dzXLnA+dbO\nuK/ttBAbBR8bscrOG/VO77+NY22HBNu4cTtbfjibgyRepU1ax98Yqs6aspHAgEC0\ncmzfDuoRFVXOEm+JktamhUv7/Nt2vsuvs7azBo738QNkmwfaarzHgw8wya4WQxGx\n505SLHMVSwsNug0DUZrbxltVfL4cTlizeAWL8VQYprfaMKuy+X18Le1b1mkrs4lr\nDgrzCZH9SXuEFXFCgfjbG6iOoI/Wi/VvhLwcuJUub5sXm91lzURKsRP8tLOmW8K8\nYwUYrgWqrUu0ZyYlAW30HSH7RLI7uQqKLSci7+rOYyqswE/wq2ylBCnmsKBPOgGX\n5izdmcfuvl+hi0fcZg3cc6VvNxd54pr0G6GP7RoBI9nsmqD7zTG6gi31DanCC2YQ\nT4mx2LrBAPEvgKEOH7bEVv4DBEwNdNlDc0ENvxPQfX6k3BMfUOBjTe4Nf+W82zok\nU1DzAo3aYA11HWSkcH73FmXyTf2txeOa6FjFyyWJcs8K7j9et6ICIeKV7UFlkE4G\n-----END RSA PRIVATE KEY-----\n")
	des3RSALongPassphrase     = []byte("-----BEGIN RSA PRIVATE KEY-----\nProc-Type: 4,ENCRYPTED\nDEK-Info: DES-EDE3-CBC,84B8B25E2A30571A\n\nJP5Gyl9YHcqCXAqiPuW1qZJ8nbvQqAr+zYcBQXkINK2oZS7jbb6YU2vIy5vGuvrK\npk38mWvhAqLh73KYexY/TnJcqtCWElnnATtl5Pb4ZtdNCZTxcbgYeO8hH/9A4BM7\nuXNhbQMNISiZMVaF3CV0kz3LGS8Rp+3lhIg5eQbPKmIwduLHPFRK1GW70lJtrnY3\nom2ERGqm3wJuFGIjdgguo2gQBEjMr6K09QuwAEdtfjYUpWFxjf5F8J37hnNXvxma\nDYXvp2CI+fvyUi5z0/BD1iE6L2l4CUMoI8bYai5WEwjbhQszL7IGNERhbm8LpVMh\n1A8XyFJiDTY+FlvDp7k4fGXhlPyN8MWFQ8Cx4tBE1cOy/xTFV4PC5Lugu+X8dfDW\n/y3v+OXNX57cpxK+sfyIye7p9P+6A0ZsaVL5znwX+lH77aAqcwLh9/TODYrpPbiK\nV/FYLx07bcc+xuH1qc63k7xxCRpfiWw8AxvF5OFbtMzS/gMwcDhUnEueJGtYeRpq\nU48AUpthtqeBQjJfHTlJ3+0XAtvSsF/BqCRCpoS/IgZiX4xliBpRIGJO9A7Eg1Uf\nF3y5dPiG3CumdqFzT3fGq5k8xQ3w3w+A0ghUOOJg+znwFp34FEp7sNi/nORvx2i8\nb9kBP1bvGE7WxxwhfgDa8ZMkD8qrXshHwcTCZT+LoY3WzZ9uGYa4xmihgW7Uwl+L\n7pWFkpFSFDxvJHTdgRLqSIz4Xjux6MLHrgm3LkbzzuvFQB65CxJ7O05V9yEgJXRH\nNufXHsARrbHqdwtzS1cIQJ00N5uP3LW6mEjfPgGIyaShHYD5TNuDZyB9qqHdIsYl\naBXtsJTIAEUraO4rdc8n133ntHRgKi+McZwgDG7QtdC2MpyZNPo0fQNyi734wNpD\nBENGK+K4iPxVYiTQP9knrY/YmXv2zvtkRfwYGVwi4fRD0uYMOhNY0nWITSf5ZmeH\nY8ilDzeNTe8QD4Q/WiG3C5FfhnfJCYZ1skJ1DEJkbirWH8lxzsOC1eBuNlBBrt9q\nmaZ2St7WXhKtvi+JXn+e/x9a8aJ+bA2Zfe29xby/VLH7+QuhKNBmJSlfsiVdiM4g\nFeq83CEcXoZy+07TKbvE+XnRwzjK8j9xBw+AuePBCAx1wL0RPTBs+vASs0Xr2edm\nTXb+gH7T5/8yL7MV9mAZMzND1y9oSCdRD8gBN0FJybidttRdUxOmAfhEBcvPfT6z\n6gmfXlyrlnFmLj6XKwxrmyxrVbMw4rQChrSC5gIARkz2y/69HhkxUft5euho7fhM\n6koNhIh6UcIW1beqSHrIjHhx96O/47RXJjGHyJro8WbmEjirYvgwhBBqumAR4aNG\nqoFVDD8gpiMT4HiWD42ePYw2W2jZhsrUyWnwcLyUtK/7fwZGygb+Mi3Lrs3K5QK9\nZaS/z0b3s9LFrQ+IT4U/CsjIS84QsPmhMEXdljNdNEMznw9SU4w/ZB0qmoO7wpco\nFnwJRrrCEwiPEPxE8j0XeyosX09G1iRsNKEFY03rMbhN/QwmDrmD8sb5ozPnSqZd\nJFRCJqnBftXOKImqaAYCMJ8SaA+ZVKkoZJgOZTuXqRKTTjL5ybczr2DKBmHogY3p\n-----END RSA PRIVATE KEY-----\n")
	unencryptedOpenSSHRSA     = []byte("-----BEGIN OPENSSH PRIVATE KEY-----\nb3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAABFwAAAAdzc2gtcn\nNhAAAAAwEAAQAAAQEAzm92mMxK1l1joueAlWuvEO26rxK6g5z0+zx5UI+fItU+BG0QCeDn\nzzjW3/TM9ZqdsRyZkW8ZVsikhQIy24FJJLcISVegcgQXCPgtMRMSkbSVTJ+bCXrCqcplIL\nN7psk4zs6af0c9SmIZ8Qns/2qvuDFIbCk/U7yV7TwtYut9+BKI5bv0TEPk1KtK3rsAaO4u\nGb2AQD8XyesIkNs0/iYeFCm+Pvdxm8zgx6mPsq4puJAUAzBqrmF5rb5fqXxJk3GAkVkE7x\nLaZeq7MTIPfiYSwOQXqQb8QCLbJJygvKHIl0vO5q2xBEd7TdSgf+f5Sw5Yv9gkVq9ywzCU\nUl+77TASoQAAA8gsUAQVLFAEFQAAAAdzc2gtcnNhAAABAQDOb3aYzErWXWOi54CVa68Q7b\nqvErqDnPT7PHlQj58i1T4EbRAJ4OfPONbf9Mz1mp2xHJmRbxlWyKSFAjLbgUkktwhJV6By\nBBcI+C0xExKRtJVMn5sJesKpymUgs3umyTjOzpp/Rz1KYhnxCez/aq+4MUhsKT9TvJXtPC\n1i6334Eojlu/RMQ+TUq0reuwBo7i4ZvYBAPxfJ6wiQ2zT+Jh4UKb4+93GbzODHqY+yrim4\nkBQDMGquYXmtvl+pfEmTcYCRWQTvEtpl6rsxMg9+JhLA5BepBvxAItsknKC8ociXS87mrb\nEER3tN1KB/5/lLDli/2CRWr3LDMJRSX7vtMBKhAAAAAwEAAQAAAQB9LjAvlepntTffKBzb\n/vGpBU+oLGY3uyz/UHuMM5tfZwn1ayFHrD9FdIUwR2UmJf0r0lj5TPKDoqXJFj06yFpPTk\nJTHzTnDr3dLYlR8evXuTazwTElfXCfv2C1hMEdsXWmSrh7J/rtqTFr7gXuGIa33gK5vAVv\nxLLxqgUnr/+yNowy8WQiZasVomtd8+g+Ikasv3UGvQHqVrnEIdUbS3DiNKfplshnmhOJ0d\njgDlCLuhzeDtFe/bwR2/8ArsjAI6xHYEpIzFB9FAFUukL3jY8Lk7+gHAECFlkibkG0g8EK\nE3QIyPEIU1TEPKckyIh/WFx2eAaAoeNCgQajqlBqPEgBAAAAgDOK63g3of1GD/L5OFRo7q\nwDc5r5t89amkmTro5gxbaeaYl9i2tDzsU1hmbDZl4y8NDWdL2pq8TpPJrsgiqmZcFKlkNT\ntg2KMm3lfFMD9BK6hOUhJaVif0clhDY2UMcAUzMCKrO879YTWVNZdj9+TPCB64Bsi7yA3v\n1vgHfZYCeuAAAAgQDrrRtQLThBl5JVoFbQragSHUWuHpaGNWH8On1eJRg2t9lHR4VV05hB\n5TZxDocSEH1GZB3EdhdLoGUQu3nPpMcN1UO08u6C8hlbsENXWsiFsQKLTiG0R69ZNaFRCC\nL4GgtLjayhrQA+sKKz90bEiHtOQpAiEWUlJLu8O+XoQTzcwQAAAIEA4DzS3dpt/JBcGzpj\nqC9PvaxH4hEnk15ocIhbWELRNXSqvPKfc6akIda5KZEl//hYhkJOzJnPZOFRwMn9L1MsT3\nQA18aOU0Ge8dkfRBeI+ihwLECJ3bbDRhkQF2fuxIwkXlhO/9WB9xq1g0RUkTf9A8jfIa7C\nApvy4bUO5n9MTeEAAAASYnJhbmRvbkBicmFuZG9uLXBjAQ==\n-----END OPENSSH PRIVATE KEY-----\n")
	openSSHRSA                = []byte("-----BEGIN OPENSSH PRIVATE KEY-----\nb3BlbnNzaC1rZXktdjEAAAAACmFlczI1Ni1jdHIAAAAGYmNyeXB0AAAAGAAAABAtsUaSj3\nOighfQ4on5IG4kAAAAEAAAAAEAAAEXAAAAB3NzaC1yc2EAAAADAQABAAABAQDTgd9fvGO/\n078PZTWyUrohFmmK5JMxEVIiFVcXXt7U8fEI1AQDHzOBqSlVSf5tKG9fdTuFWyT/GPBEkd\nltQPcCpW0S20YT2J73skq+dkOwnax6H/96xZVhLBpuf2hB2F1FkC2FQ85HIRU30Y9FXcYV\nF1eki5TpnhDnZy+h4ocPwcnchmw5ZFIrW+MeC2SPtapHGN9apGlTuGwtm5Nmx+mWAzjb3g\nNAhsbu336JyvhSaN2UOVqmJAHNy2+NhRr5UE1e8eObU7/72zX8sxH+2DoKQO8ppOYTPjAf\nEpxoY8EkPjIVZNKmKYVsBovFr7zSPZ7Z9O3ZCg/VU4e5dB+10V3hAAAD0P8MHHidQXQYDd\nJasGAA9GzWCNMxGfBhtvkcwsIEBNMPlVNQ3GDkKiUSozHclp4sFMK2whKaJ/znLd0vt/fP\n4I3h6Y9SeQv64f1+Ss8HqT20lFMUyc2D6/7lCq/PYJ/ryEWa0Ov+vsiZcjiHUqEXOjTXQw\n9NE2FK86OmP1q6QahoE9wEIts7hnKisWGTSUMtFfFpkXQWiZ/2ryn53pqJrsZ6Tqs+RTMC\nLZuN9m4AqDlJ3geUDHsd0pRp4XDnHK4Epv1VB2uyg7Uz/CYgoybxBFQYoD/FlNf9jkbDnj\nJX1ry0Ds+kvB95kU3C6RZ7VXDI8cSRzDYx+IekAPLMYD8wto4+fwDGns1uGeVzIboIzdB9\natr+Vnmo2fDoWOVa8NY9RmfMh7etTVW1WkUbbLRf49x3tBnF43bX9bVJZz7197cNX7L7hg\ntKF1wCXiNEPrgUTU0anhD9PuOG52wl4BUOOhnW9yY2s+3IR9Sl2BMPwh6LcKG17c4nwmPb\nKv32hHdXZeQssOMuYHdPOwTT/3rm/HprxR7XBKaghMbnnWGZYhxM6qILP/FddFLoIUZYt+\nU3am5u/4WHKErmAogWTXl1hU5mkuuFapaaGAIr+3ZKHv0Do+Dp0uu6dD3cTkRgYwj2Ubi5\nUyT/L2RWn7MO/1/fYd6hu8pnqCGq2qc37iBlmUxuLur4aUskNTNXj6wVUc2CoqxmDmnp/O\ndtTBZnrqfkVEVrkWxn3foBilNsm3ifvymI8JC1r4EEtjVPyug8YicVqHZia2mwr9ESaf64\nh2GdyeQbUNE9lZnYNrcc6Fb0FwKIV6HpT5gOOw5WWEe2OFmGm/RqdqrN97oHAq82Ga63n/\nYeAyugQNVY7z+7lWib1KaDdrAENhVhzT4BSN05WTSMbzqxOUgR/vRogZFODDzVovlbPx4J\ncZfZojeZ29K4gm06E+QsZ2oEMrLkBpv9tMslylqfjsCQJ0iKZ4ghzoj6Lk75SgvdSymu24\nJyC+czlq3VNDXc3MVMIHxG5LlPAhS10FXjGh5NGIkpGf3tLQ5F1dJ3yu3XB5TqcgmwFaTu\nPr18z/u0bvNHR8WspgQVmk8u+RxcxiFBdM5FkjD2sDlQ0MSiIqDoSulxeYcvFjQZ72U6We\ncUzWHJ51oLGLxrOGZWuw4vRlAlJQMVM1G+rB18NGXiVIwkQc+z2TIKsiwgUnOPo0Wd2Iaf\nebF/EHc4h1PRsgwmK3t2/awj3u1S8/x41lqVNXn0POEY88XR8YKbGJ8v6FZq4aMSzmefT1\nAKxBh0f+1/EjgqoJZVi7k1xDh2vpU=\n-----END OPENSSH PRIVATE KEY-----\n")
	unencryptedOpenSSHecdsa   = []byte("-----BEGIN OPENSSH PRIVATE KEY-----\nb3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAAAaAAAABNlY2RzYS\n1zaGEyLW5pc3RwMjU2AAAACG5pc3RwMjU2AAAAQQRMztCVZh7L86rQ1AxpT3R0eZkksQ02\ndAoCPrUWKfcVRg6cDKGEO6Hx8fEVBgUyF9nhQdOUMOkDbGw7YRkE2AQVAAAAsOppCH3qaQ\nh9AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBEzO0JVmHsvzqtDU\nDGlPdHR5mSSxDTZ0CgI+tRYp9xVGDpwMoYQ7ofHx8RUGBTIX2eFB05Qw6QNsbDthGQTYBB\nUAAAAhAJM8EXIZ1ZnmGeka8yqKdW9Ux5jsIKcdUfYbpLyhp9+kAAAAEmJyYW5kb25AYnJh\nbmRvbi1wYwECAwQF\n-----END OPENSSH PRIVATE KEY-----\n")
	openSSHecdsa              = []byte("-----BEGIN OPENSSH PRIVATE KEY-----\nb3BlbnNzaC1rZXktdjEAAAAACmFlczI1Ni1jdHIAAAAGYmNyeXB0AAAAGAAAABCJlmRAXN\nbd9/lx980iAX6RAAAAEAAAAAEAAABoAAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlz\ndHAyNTYAAABBBGkauFKRKZzxzOdzkeGKUVUKtj0rhaAQVmeSAMe+sLu/gkxVUsvd0DrUWR\nQMBErnrDug25HqhoTxPrU3Bck3htcAAACwV+zzPxsGuGkb3VX2DH3tMSuFMdG4TPAlZ1PD\ndY3wOFI4u679zGwZI6xyyR5ra0ZIuOuEuJuef+f78IMR0Ji+ohxKjyqbUrnBF/7CscImNM\nBd4k3nAuAeh90k717z0sH7sxIjrWiVFcgR43iDDk2EExrW7a1hKDeW0YcvNOhxFfHqt5nz\nVmGTBC4yLNeJ808eNmBcSm4bLGyJMHZ2UHJwFF+4F4piqoiUfhV24/7zUcA=\n-----END OPENSSH PRIVATE KEY-----\n")
	unencryptedOpenSSHed25519 = []byte("-----BEGIN OPENSSH PRIVATE KEY-----\nb3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAAAMwAAAAtzc2gtZW\nQyNTUxOQAAACB/Glz+CgmDtZ12iEB+Pvs9IYDKJqKdzduwDBVa7DjEpwAAAJg8Vw4WPFcO\nFgAAAAtzc2gtZWQyNTUxOQAAACB/Glz+CgmDtZ12iEB+Pvs9IYDKJqKdzduwDBVa7DjEpw\nAAAEDFsXsKIQHwRfg4D/TLQllujVFnpppk+FkSj7ESylb3d38aXP4KCYO1nXaIQH4++z0h\ngMomop3N27AMFVrsOMSnAAAAEmJyYW5kb25AYnJhbmRvbi1wYwECAw==\n-----END OPENSSH PRIVATE KEY-----\n")
	openSSHed25519            = []byte("-----BEGIN OPENSSH PRIVATE KEY-----\nb3BlbnNzaC1rZXktdjEAAAAACmFlczI1Ni1jdHIAAAAGYmNyeXB0AAAAGAAAABCKEv5TW3\n2/+Yrh5NHKSiHaAAAAEAAAAAEAAAAzAAAAC3NzaC1lZDI1NTE5AAAAILsonADZ8AGZFUGY\nEqF/B8YeopQOhQw7bAbizUwC//NTAAAAoORLKIJavH35i2ziohvCmkhGP7zoPpGnuH7skJ\nRlbu8LOgjcL47xJgkcl7Xvw4FPYn2y2bdPj/l8GyEbuxV/ws4A1p55jy5jfaA/wPOaEaOn\nfOMZK/hxlSkYyaVgaWu188UnEXnpFnKTPylcsDEBG+k2E2416wPe63/1ptGyuYvpmM/BTr\n5UrVcKJK1bwxwCMwSDmvsSll4fETyWZtNROTI=\n-----END OPENSSH PRIVATE KEY-----\n")
)

func TestReadPrivateKeyWithContent(t *testing.T) {
	cases := []struct {
		Name              string
		PrivateKeyContent []byte
		Expect            func(t *testing.T, signer ssh.Signer, err error)
	}{
		{
			Name:              "Invalid Key",
			PrivateKeyContent: invalidKey,
			Expect: func(t *testing.T, signer ssh.Signer, err error) {
				require.Error(t, err)
				require.Nil(t, signer)
				require.Contains(t, err.Error(), "PEM")
			},
		},
		{
			Name:              "Unencrypted RSA Key",
			PrivateKeyContent: unencryptedRSAKey,
			Expect: func(t *testing.T, signer ssh.Signer, err error) {
				require.Nil(t, err)
				require.NotNil(t, signer)
			},
		},
		{
			Name:              "Unencrypted OpenSSH RSA Key",
			PrivateKeyContent: unencryptedOpenSSHRSA,
			Expect: func(t *testing.T, signer ssh.Signer, err error) {
				require.Nil(t, err)
				require.NotNil(t, signer)
			},
		},
		{
			Name:              "Unencrypted OpenSSH ecdsa Key",
			PrivateKeyContent: unencryptedOpenSSHecdsa,
			Expect: func(t *testing.T, signer ssh.Signer, err error) {
				require.NoError(t, err)
				require.NotNil(t, signer)
			},
		},
		{
			Name:              "Unencrypted OpenSSH ed25519 Key",
			PrivateKeyContent: unencryptedOpenSSHed25519,
			Expect: func(t *testing.T, signer ssh.Signer, err error) {
				require.Nil(t, err)
				require.NotNil(t, signer)
			},
		},
	}

	for _, c := range cases {
		t.Run(c.Name, func(t *testing.T) {
			signer, err := readPrivateKeyWithContent(context.Background(), "n/a", c.PrivateKeyContent)

			c.Expect(t, signer, err)
		})
	}
}
